<?php
/**
 * Rest API class file for the Virusdie Plugin.
 *
 * @package Virusdie Plugin
 */

// Make sure the file is not directly accessible.
if ( ! defined( 'ABSPATH' ) ) {
	die( 'We\'re sorry, but you can not directly access this file.' );
}
class VDWS_VirusdieApiClient
{

	private static $connType;

	public function __construct()
	{
		static::init();
	}

	public static function init()
	{
		self::check_conn_type();
	}

	private static function build_stream_context($method, $apikey, array $vars = array(), $json = false )
	{
		if ( !is_string($method) || !is_string($apikey) || !in_array($method, array('GET', 'POST', 'PUT', 'HEAD', 'PATCH') ) )
			return false;

		$conn_type = self::get_conn_type();
		if ( $conn_type === 'php') {
			return self::php_build_stream_context($method, $apikey, $vars, $json);
		} elseif ($conn_type === 'curl') {
			return self::curl_build_stream_context($method, $apikey, $vars, $json);
		} else {
			return false;
		}
	}

	private static function make_http_request( $url, $context )
	{
		$conn_type = self::get_conn_type();
		if ( $conn_type === 'php') {
			return self::php_make_http_request( $url, $context );
		} elseif ($conn_type === 'curl') {
			return self::curl_make_http_request( $url, $context );
		} else {
			return false;
		}
	}

	private static function php_build_stream_context($method, $apikey, array $vars = array(), $json = false )
	{
		if ( !is_string($method) || !is_string($apikey) || !in_array($method, array('GET', 'POST', 'PUT', 'HEAD', 'PATCH') ) )
			return false;

		$ctype = $json ? "application/json" : "application/x-www-form-urlencoded";
		$request = array(
			'http' => array(
				'method' => $method,
				'header' => "Accept-language: en\r\n" .
					"Cookie: apikey={$apikey}\r\n" . 
					"Content-Type: {$ctype}; charset=UTF-8\r\n",
				//'follow_location' => 0,
				'content' => $json ? sanitize_textarea_field(json_encode($vars)) : http_build_query($vars)
			)
		);
		return stream_context_create($request);
	}

	private static function curl_build_stream_context($method, $apikey, array $vars = array(), $json = false )
	{
		if ( !is_string($method) || !is_string($apikey) || !in_array($method, array('GET', 'POST', 'PUT', 'HEAD', 'PATCH') ) )
			return false;

		$ctype = $json ? "application/json" : "application/x-www-form-urlencoded";

		$headers = [];
		$headers[] = 'Cookie: apikey=' . $apikey;;
		$headers[] = 'Content-Type: ' . $ctype;
	
		$ch = curl_init(); 
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
		curl_setopt($ch, CURLOPT_HEADER, 0);
		curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
		curl_setopt($ch, CURLOPT_TIMEOUT, 10);
	
		if (in_array($method, array('POST', 'PUT', 'PATCH'))) {
			curl_setopt($curl, CURLOPT_POST, true);
			curl_setopt($curl, CURLOPT_POSTFIELDS, http_build_query($vars));
		}
		return $ch;
	}

	private static function php_make_http_request($url, $context)
	{
		return @json_decode( file_get_contents( $url, false, $context), true );
	}

	private static function curl_make_http_request($url, $context)
	{
		curl_setopt($context, CURLOPT_URL, $url);
		$result = curl_exec($context);
		curl_close($context);
		return @json_decode( $result, true );
	}

	private static function check_result( $result, &$error = null )
	{
		if ( !is_array($result) || !key_exists('error', $result) || !key_exists('result', $result) || $result['error'] != 0 /*!in_array($result['error'], array(0, 111901) )*/ ) {
			$error = $result ? $result['error'] : -1;
			return false;
		} else {
			return true;
		}
	}

	public static function signup( $email, &$error = null)
	{
		if ( !is_string($email) )
			return false;
		 
		$url = constant('VDWS_VIRUSDIE_API_HOST') . 'signup/' . $email;

		$context = self::build_stream_context('GET', constant('VDWS_VIRUSDIE_SHARED_KEY') );

		return self::check_result( self::make_http_request($url, $context), $error );
	}

	public static function signin( $email, $code )
	{
		if ( !is_string($email) || !is_string($code) )
			return false;

		$url = constant('VDWS_VIRUSDIE_API_HOST') . 'signin/' . $email . '/' . $code;

		$context = self::build_stream_context('GET', constant('VDWS_VIRUSDIE_SHARED_KEY') );
		$result = self::make_http_request($url, $context);

		return self::check_result($result) ? $result['result']['privatekey']['key'] : false;
	}

	public static function signout()
	{
		$url = constant('VDWS_VIRUSDIE_API_HOST') . 'signout/';

		$context = self::build_stream_context('GET', VDWS_Virusdie::get_api_key() );

		return self::check_result( self::make_http_request($url, $context) ) && VDWS_Virusdie::del_api_key();
	}

	public static function get_dashboard_link()
	{
		$url = constant('VDWS_VIRUSDIE_API_HOST') . 'dashboard';

		$context = self::build_stream_context('GET', VDWS_Virusdie::get_api_key() );

		$result = self::make_http_request($url, $context);

		return self::check_result( $result ) ? $result['result'] : false;
	}

	public static function get_site_info()
	{
		$url = constant('VDWS_VIRUSDIE_API_HOST') . 'sites_list?withscanreport&withscanhistory&withfwhistory';

		$context = self::build_stream_context('GET', VDWS_Virusdie::get_api_key() );

		$result = self::make_http_request($url, $context);

		return self::check_result( $result ) ? $result['result'] : false;
	}

	public static function get_user_info()
	{
		$url = constant('VDWS_VIRUSDIE_API_HOST') . 'api_userinfo/';

		$context = self::build_stream_context('GET', VDWS_Virusdie::get_api_key() );

		$result = self::make_http_request($url, $context);

		return self::check_result( $result ) ? $result['result'] : false;
	}

	public static function add_site()
	{
		$url = constant('VDWS_VIRUSDIE_API_HOST') . 'sites_add/';

		$context = self::build_stream_context( 'POST', VDWS_Virusdie::get_api_key(), array(VDWS_Virusdie::get_domain()), true );

		$result = self::make_http_request($url, $context);

		self::check_result( $result, $error );

		return $error;
	}

	public static function get_site_count()
	{

	}

	public static function is_key_valid()
	{
		$url = constant('VDWS_VIRUSDIE_API_HOST') . 'api_keyinfo';
		if (!($key = VDWS_Virusdie::get_api_key()))
			return false;

		$context = $key ? self::build_stream_context('GET', $key ) : null;;

		$result = self::make_http_request($url, $context);

		return self::check_result( $result ) && $result['result']['expires'] > time();
	}

	public static function getSyncFile( VDWS_VirusdieUser $user )
	{
		$url = constant('VDWS_VIRUSDIE_API_HOST') . 'syncfile_get';

		$context = self::build_stream_context('GET', VDWS_Virusdie::get_api_key() );

		$fcontent = file_get_contents($url, false, $context);
		$fname = $user->getSyncFileName();

		if ( $fcontent && $fname ) {
			return array(
				'name' => $fname,
				'content' => $fcontent
			);
		} else {
			return false;
		}
	}

	public static function scan()
	{
		$url = constant('VDWS_VIRUSDIE_API_HOST') . 'site_scan/' . VDWS_Virusdie::get_domain() . '/';

		$context = self::build_stream_context('GET', VDWS_Virusdie::get_api_key() );

		$result = self::make_http_request($url, $context);

		return self::check_result( $result ) ? true : false;
	}

	public static function change_option( $option, $value )
	{
		if ( !is_string($option) ) 
			return false;

		$url = constant('VDWS_VIRUSDIE_API_HOST') . 'site_setoption/' . VDWS_Virusdie::get_domain() . '/' . $option . '/' . $value . '/';

		$context = self::build_stream_context('GET', VDWS_Virusdie::get_api_key() );

		$result = self::make_http_request($url, $context);

		return self::check_result( $result, $error ) && !$error ? $result['result'] : false;
	}

	private static function is_curl_enable() 
	{
		if (in_array ('curl', get_loaded_extensions())) {
			return true;
		} else {
			return false;
		}
	}

	private static function is_fopen_enable()
	{
		return ini_get('allow_url_fopen');
	}


	private static function check_conn_type()
	{
		if (self::is_fopen_enable()) {
			self::$connType = 'php';
		} elseif (self::is_curl_enable()) {
			self::$connType = 'curl';
		} else {
			self::$connType = false;
		}
	}

	public static function get_conn_type()
	{
		return self::$connType;
	}
}
