<?php
/**
 * Helper class file for the Virusdie Plugin Messages.
 *
 * @package Virusdie Plugin
 */

// Make sure the file is not directly accessible.
if ( ! defined( 'ABSPATH' ) ) {
	die( 'We\'re sorry, but you can not directly access this file.' );
}

class VDWS_VirusdieMessages
{
	private static $messages;
	private static $markers;
	private static $user;
	private static $site;


	public function  __construct( $user, $site = null)
	{
		static::init( $user, $site );
	}

	public static function init( $user, $site )
	{
		self::$messages = array();
		self::$markers = array();
		self::$user = $user;
		self::$site = $site;
		self::messagesCreation();
		self::coloredMarkersCreation();
	}

	public static function getTextMessage( $msg_name )
	{
		return is_string($msg_name) && isset(self::$messages[$msg_name]) ? esc_html(self::$messages[$msg_name]) : '';
	}

	private static function setTextMessage( $msg_name, $msg_text )
	{
		if ( !is_string($msg_name) && !is_string($msg_text) )
			return false;

		self::$messages[$msg_name] = sanitize_text_field($msg_text);
	}

	public static function getMarkerColor( $marker_name )
	{
		if ( !is_string($marker_name) )
			return false;

		return esc_attr(strval(self::$markers[$marker_name]));
	}

	private static function setMarkerColor( $marker_name, $color )
	{
		if ( !is_string($marker_name) || !is_string($color) )
			return false;

		self::$markers[$marker_name] = sanitize_key($color);
	}

	private static function messagesCreation()
	{
		self::setTextMessage('scanned_at', 'Scanned at: ' . date('M d H:i', VDWS_VirusdieHelper::getBrowserTime(self::$site->lastScan()) ) );
		self::setTextMessage('head_status', self::$site->isSyncError() ? 
				'Sync problem. <a href="https://virusdie.com/websites/'. self::$user->getLoginParams() . '" class="vd-mainBlock__info-link --not-sync" target="_blank">Fix sync problem</a>' : ( (
				self::$site->getInfectedCount() ||  
				self::$site->getVulCount() || 
				self::$site->getDbCount() || 
				self::$site->getFirewallBlockedCount() ||
				self::$site->getFirewallBlockedIp() ||
				self::$site->isBlacklisted() ) ?  
				'Issues were found' : 'No issues found') );
		self::setTextMessage('fw_report_date', 'Report ' . date('M d', self::$site->lastScan() ) );
		self::setTextMessage('scan_status', self::$site->getInfectedCount() ? 
			self::$site->getInfectedCount() . ' infected '. self::Declension('files', self::$site->getInfectedCount()) . ' found' : 'No threats found');
		self::setTextMessage('vul_status', self::$site->getVulCount() ? 
			self::$site->getVulCount() . ' ' . self::Declension('vulnerabilities', self::$site->getVulCount()) . ' found' : 'No threats found');
		self::setTextMessage('db_status', self::$site->getDbCount() ? 
			self::$site->getDbCount() . ' No threats found' : 'No threats found');
		self::setTextMessage('fw_status', self::$site->getFirewallBlockedCount() || self::$site->getFirewallBlockedIp() ? 
			self::$site->getFirewallBlockedCount() . ' ' . self::Declension('attacks were', self::$site->getFirewallBlockedCount()) . ' blocked from ' . 
				self::$site->getFirewallBlockedIp() .' IP ' . self::Declension('addresses', 	self::$site->getFirewallBlockedIp()) . ' today' : 
					'No attacks were blocked today');
		self::setTextMessage('fw_status_f', self::$site->getFirewallBlockedCount() || self::$site->getFirewallBlockedIp() ?
			self::$site->getFirewallBlockedCount() . ' ' . self::Declension('attacks were', self::$site->getFirewallBlockedCount()) . ' detected from ' .
				self::$site->getFirewallBlockedIp() . ' IP ' . self::Declension('addresses', 	self::$site->getFirewallBlockedIp()) . ' today' :
					'No attacks were detected today');
		self::setTextMessage('black_status', self::$site->isBlacklisted() ? 
			'Mentioned on '. self::$site->getBlacklistedCount() . ' ' . self::Declension('blacklists', self::$site->getBlacklistedCount()) : 'The site is not blacklisted');
	}

	private static function coloredMarkersCreation()
	{
		self::setMarkerColor('sync_status', self::$site->isSyncError() ? '--not-sync' : '--clear');
		self::setMarkerColor('scan_status', self::$site->getInfectedCount() ? '--malware' : '--clear');
		self::setMarkerColor('vul_status', self::$site->getVulCount() ? '--vulnerable' : '--clear');
		self::setMarkerColor('db_status', self::$site->getDbCount() ? '--database' : '--clear');
		self::setMarkerColor('fw_status', self::$site->getFirewallBlockedIp() ? '--malware' : '--clear');
		self::setMarkerColor('black_status', self::$site->isBlacklisted() ? '--blacklisted' : '--clear');
	}

	private static function Declension( $word, $num )
	{
		$res = '';
		switch ($word) {
			case 'files':
				$res = $num == 1 ? 'file' : 'files';
				break;
			case 'vulnerabilities':
				$res = $num == 1 ? 'vulnerability' : 'vulnerabilities';
				break;
			case 'attacks were':
				$res = $num == 1 ? 'attack was' : 'attacks were';
				break;
			case 'addresses': 
				$res = $num == 1 ? 'address' : 'addresses';
				break;
			case 'blacklists': 
				$res = $num == 1 ? 'blacklist' : 'blacklists';
				break;
			default:
				$res = $word;
		}
		return $res;
	}

}