<?php
/**
 * Site class file for the Virusdie Plugin.
 *
 * @package Virusdie Plugin
 */

// Make sure the file is not directly accessible.
if ( ! defined( 'ABSPATH' ) ) {
	die( 'We\'re sorry, but you can not directly access this file.' );
}

class VDWS_VirusdieSite
{
	private $id = 0;
	private $domain = '';
	private $firewall_status = 0;
	private $autotreatment_status = 0;
	private $dailyscan_status = 0;
	private $last_scan = 0;
	private $firewall_blocked = array();
	private $patchmanager_status = 0;
	private $scanreport_id = 0;
	private $sync_err = false;
	private $blstatus = false;

	private $detected_m = 0;
	private $cleaned_m = 0;
	private $deleted_m = 0;
	private $detected_v = 0;
	private $cleaned_v = 0;
	private $deleted_v = 0;

	private $error = 0;

	public $checked = 0;

	public function __construct( VDWS_VirusdieUser $user )
	{
		$this->init($user);
	}

	private function init( VDWS_VirusdieUser &$user )
	{
		$sites = VDWS_VirusdieApiClient::get_site_info();
		$user->setSitesCount(count($sites));
		foreach( $sites as $site) {
			if (!empty(array_uintersect(array($site['host'], $site['domain']), array($_SERVER['HTTP_HOST'], $_SERVER['SERVER_NAME']), 'strcasecmp'))) {
				$user->hasCurrentSite(true);
				$this->id = intval($site['id']);
				$this->domain = strval($site['domain']);
				$this->firewall_status = boolval($site['firewallstatus']);
				$this->autotreatment_status = boolval($site['autotreatment']);
				$this->dailyscan_status = boolval($site['scanperiod'] <= 86400 && intval($site['scanperiod']) !== 0);
				$this->patchmanager_status = $user->isPaid() ? boolval($site['firewallstatus']) : 
					(boolval($site['autopatching']) && boolval($site['firewallstatus']));
				$this->blstatus = boolval($site['blstatus']);
				$this->blacklisted = boolval($this->blstatus) ? count(json_decode($site['blacklisted'], true)) : 0;

				if ( isset($site['scanreport']) && is_array($site['scanreport'] ) ) {
					$this->last_scan = intval($site['scanreport']['date']);
					$this->scanreport_id = intval($site['scanreport']['id']);
					$this->detected_m = intval($site['scanreport']['detected_m']);
					$this->cleaned_m = intval($site['scanreport']['cleaned_m']);
					$this->deleted_m = intval($site['scanreport']['deleted_m']);
					$this->detected_v = intval($site['scanreport']['detected_v']);
					$this->cleaned_v = intval($site['scanreport']['cleaned_v']);
					$this->deleted_v = intval($site['scanreport']['deleted_v']);
					$this->error = boolval($site['scanreport']['errors']);
				}

				if ( isset($site['scanhistory']) && is_array($site['scanhistory']) ) {
					$this->sync_err = reset($site['scanhistory']) === 'not-synced';
				}

				$this->firewall_blocked = VDWS_VirusdieHelper::getBlockedIp($user, $site);

				$this->checked = intval($this->dailyscan_status ? ++$this->checked : $this->checked);
				$this->checked = intval($this->autotreatment_status ? ++$this->checked : $this->checked);
				$this->checked = intval($this->firewall_status ? ++$this->checked : $this->checked);
				$this->checked = intval($this->patchmanager_status ? ++$this->checked : $this->checked);
				$this->checked = intval($this->blacklisted ? ++$this->checked : $this->checked);

				$_SESSION['vdws_domain'] = $this->getDomain();
				$this->jsSiteId();

				break;
			}
		}
	}

	public function getId()
	{
		return intval($this->id);
	}

	public function getDomain()
	{
		return strval($this->domain);
	}

	public function lastScan()
	{
		return intval($this->last_scan);
	}

	public function isDailyScan()
	{
		return boolval($this->dailyscan_status);
	}

	public function isAutoTreatment()
	{
		return boolval($this->autotreatment_status);
	}

	public function getInfectedCount()
	{
		return intval($this->detected_m - ( $this->cleaned_m + $this->deleted_m ));
	}

	public function getVulCount()
	{
		return intval($this->detected_v - ( $this->cleaned_v + $this->deleted_v ));
	}

	public function getDbCount()
	{
		return intval(0);
	}

	public function isBlacklisted()
	{
		return boolval($this->blstatus);
	}

	public function getBlacklistedCount()
	{
		return intval($this->blacklisted);
	}

	public function isPatchManager()
	{
		return boolval($this->patchmanager_status);
	}

	public function isSyncError()
	{
		return boolval($this->error) && boolval($this->sync_err);
	}

	public function isScanned()
	{
		return boolval(true);
	}

	public function isFirewallOn()
	{
		return boolval($this->firewall_status);
	}

	public function getFirewallBlockedCount()
	{
		return isset($this->firewall_blocked['total']) ? intval($this->firewall_blocked['total']) : false;;
	}

	public function getFirewallBlockedIp()
	{
		return isset($this->firewall_blocked['ip']) ? $this->firewall_blocked['ip'] : false;
	}

	public function getFirewallBlockedIso() 
	{
		return isset($this->firewall_blocked['iso']) ? $this->firewall_blocked['iso'] : false;
	}

	public function getScanreportId() 
	{
		return intval($this->scanreport_id);
	}

	public function jsSiteId()
	{
		echo '<script>' .
			' var VDWS_SITE_ID = ' . $this->getId() . ';' .
			' </script>' . 
			"\n";
	}

}